#!/usr/bin/env python3.1

# Change the following string to point to the correct path.
SEXP3_FOLDER       = '/home/aortiz/weblang/sexp3'

PORT               = 8003
HOST_NAME          = 'localhost'
VERSION            = '0.1'
SERVER_SOFTWARE    = 'Sexp-WebLang/{0}'.format(VERSION)
TEMPLATES_FOLDER   = SEXP3_FOLDER + '/templates'
OTHERS_FOLDER      = SEXP3_FOLDER + '/others'

import socket
import datetime
from template import get_file_str
from email.utils import formatdate

#-------------------------------------------------------------------------------
def run_server():

    print(SERVER_SOFTWARE)
    print('Development server is running at http://{0}:{1}/'.format(HOST_NAME, PORT))
    print('Quit the server with CONTROL-C.')

    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.bind((HOST_NAME, PORT))
    sock.listen(1)   

    try:
        while True:                       
            new_sock, address = sock.accept()                           
            try:
                raw_info = new_sock.recv(1 << 20)
                environ, request_line = parse_input(raw_info)                
                add_server_variables(environ)
                
                status, content_type, response_body = get_file_str(environ, TEMPLATES_FOLDER, OTHERS_FOLDER)

                print('{0} {1} "{2}" {3} {4}'.format(
                    address[0], 
                    datetime.datetime.now().strftime('[%d/%b/%Y %H:%M:%S]'),
                    request_line,
                    status.rsplit(' ')[0],
                    len(response_body)))
                
                if type(response_body) == str:
                    response_body = response_body.encode('UTF-8') 
                                
                new_sock.send(headers(status, content_type, len(response_body)).encode('UTF-8') + 
                    response_body)                
            finally:
                new_sock.close()
    finally:
        sock.close()
        
#-------------------------------------------------------------------------------        
def add_server_variables(environ):
    environ['SERVER_SOFTWARE'] = SERVER_SOFTWARE
    environ['SERVER_PROTOCOL'] = 'HTTP/1.1'
    environ['SERVER_PORT'] = str(PORT)
    environ['SERVER_NAME'] = HOST_NAME        
    
#-------------------------------------------------------------------------------
def headers(status, content_type, content_length):
    return \
'''HTTP/1.1 {0}
Date: {1}
Server: {2}
Content-Length: {3}
Content-type: {4}

'''.format(status, 
           formatdate(timeval=None, localtime=False, usegmt=True), 
           SERVER_SOFTWARE, 
           content_length,
           content_type)

#-------------------------------------------------------------------------------
def parse_input(raw_info):
    result = {}
    str_info = raw_info.decode('UTF-8').rsplit('\r\n')
    
    # Parse request line.
    (result['REQUEST_METHOD'], 
     request_uri_qs, 
     request_protocol) = str_info[0].rsplit(' ')
     
    (result['PATH_INFO'],
     result['QUERY_STRING']) = \
     request_uri_qs.rsplit('?') if '?' in request_uri_qs else (request_uri_qs, '')  
     
    # Parse headers until first empty line.
    i = 1
    while str_info[i]:
        (header_name, header_value) = str_info[i].split(': ')
        result['HTTP_{0}'.format(header_name.upper().replace('-', '_'))] = header_value        
        i += 1
        
    # The query string is in the body of a POST request.     
    if result['REQUEST_METHOD'] == 'POST':
        result['QUERY_STRING'] = str_info[i + 1]                
    
    return result, str_info[0]
    
#-------------------------------------------------------------------------------        
run_server()
