from re import compile
from urllib.parse import parse_qs
from sexp import interpret
from os.path import isfile

#-------------------------------------------------------------------------------
regex = compile(r'{{(.*)}}|(.|\n)')

def eval_template(file_name, environ):
    
    new_environ = environ.copy()
    new_environ.update(qs_dict(environ.get('QUERY_STRING', '')))
    
    response_body = []
    
    with open(file_name, encoding='UTF-8') as f:
        
        first_line = f.readline()        
        data = f.read()
        
        if not first_line.startswith("#!"):
            data = first_line + data
        
        for m in regex.finditer(data):
            if m.group(1):
                try:
                    output = interpret(m.group(1), new_environ)                    
                    response_body.append(str(output))                                        
                except Exception as e:
                    response_body.append(str(e))                    
            else:
                response_body.append(m.group())
                
    return ''.join(response_body)                 
                     
#-------------------------------------------------------------------------------                     
def qs_dict(query_string):        
    params = {}
    qdict = parse_qs(query_string)
    for k in qdict:
        value = qdict[k][0]
        params[k] = int(value) if value.isdigit() else value 
    return params

#-------------------------------------------------------------------------------
content_type = {
    'html' : 'text/html; charset=UTF-8',
    'sexp' : 'text/html; charset=UTF-8',
    'txt'  : 'text/plain; charset=UTF-8',
    'css'  : 'text/css; charset=UTF-8',
    'js'   : 'text/javascript; charset=UTF-8',
    'gif'  : 'image/gif',
    'jpg'  : 'image/jpeg',
    'png'  : 'image/x-png',        
}

def get_file_str(environ, templates_folder, others_folder):

    path_info = environ['PATH_INFO']
    
    if path_info == '/':
        path_info = '/index.sexp'
        
    ext_index = path_info.rfind('.')
    extension = '' if ext_index == -1 else path_info[ext_index + 1:]    
        
    file_name = (templates_folder if extension == 'sexp' 
                 else others_folder) + path_info    
        
    if isfile(file_name):
        status = '200 OK'        
    else:
        status = '404 Not Found'        
        file_name = templates_folder + '/404.sexp'
        extension = 'sexp'
        
    if extension == 'sexp':
        response_body = eval_template(file_name, environ)        
    else:
        with open(file_name, 'rb') as f:
            response_body = f.read()
            
    return status, content_type.get(extension, 'application/octet-stream'), response_body

